@@ -1,16 +1,14 @@
 import os
-from abc import ABCMeta, abstractmethod
+from abc import ABC, abstractmethod
 from contextlib import contextmanager
 
-from six import add_metaclass
+from typing import IO, Generator
 
 
-@add_metaclass(ABCMeta)
-class DataFile:
-    """ A datafile that holds the source path and method to get a handle
-    """
+class DataFile(ABC):
+    """A datafile that holds the source path and method to get a handle"""
 
-    def __init__(self, path):
+    def __init__(self, path: str):
         self.path = path
 
     @abstractmethod
@@ -19,12 +17,12 @@ class DataFile:
 
 
 class LocalDataFile(DataFile):
-    def __init__(self, path, local):
+    def __init__(self, path: str, local: str):
         self.local = local
         super(LocalDataFile, self).__init__(path)
 
     @contextmanager
-    def handle(self, mode="rb"):
+    def handle(self, mode: str = "rb") -> Generator[IO, None, None]:
         if mode.startswith("w") and not os.path.isdir(os.path.dirname(self.local)):
             os.makedirs(os.path.dirname(self.local))
         with open(self.local, mode) as f:
