@@ -1,6 +1,5 @@
 import gzip
 import os
-import six
 import warnings
 import pandas as pd
 from io import TextIOWrapper
@@ -14,7 +13,7 @@ except ImportError:
 
 
 def read_df(datafile, **read_args):
-    """ Read a dataframe from file based on the file extension
+    """Read a dataframe from file based on the file extension
 
     The following formats are supported:
     parquet, avro, csv, pickle
@@ -58,7 +57,7 @@ def read_df(datafile, **read_args):
 
 
 def write_df(df, datafile, **write_args):
-    """ Write a dataframe to file based on the file extension
+    """Write a dataframe to file based on the file extension
 
     The following formats are supported:
     parquet, avro, csv, pickle
@@ -113,15 +112,14 @@ def write_df(df, datafile, **write_args):
                 "compression"
             ] = None  # default "infer" incompatible with handles as of 0.24
 
-        if manual_compress and compression == "gzip":
-            buf = gzip.GzipFile(fileobj=buf, mode="w")
-        elif manual_compress and compression is not None:
+        if manual_compress and compression is not None:
             raise ValueError(
-                "Compression {} is not supported for CSV/Pickle".format(compression)
+                "Compression {} is not supported for CSV/Pickle/JSON".format(
+                    compression
+                )
             )
 
-        # And for 23 compatibility need a textiowrapper for text formats
-        if write_name in ("to_csv", "to_json") and six.PY3:
+        if write_name in ("to_csv", "to_json"):
             buf = TextIOWrapper(buf, write_through=True)
 
         write_fn = getattr(df, write_name)
