@@ -4,7 +4,7 @@ import blocks
 
 
 def test_place(datadir, randomdata, fs):
-    blocks.place(randomdata, os.path.join(datadir, "example.csv"))
+    blocks.place(randomdata, os.path.join(datadir, "example.parquet"), filesystem=fs)
     assert len(fs.ls(datadir)) == 1
-    df = blocks.assemble(datadir)
+    df = blocks.assemble(datadir, filesystem=fs)
     assert np.isclose(df, randomdata).all().all()
